package com.futurescale.sa.model.proxy
{
	import com.futurescale.sa.model.vo.ValueObject;
	
	import flash.filesystem.File;
	import flash.filesystem.FileMode;
	import flash.filesystem.FileStream;
	
	import mx.collections.ArrayCollection;
	
	import org.puremvc.as3.patterns.proxy.Proxy;
	
	import spark.collections.Sort;
	import spark.collections.SortField;
	
	/**
	 * Provides subclasses with a simple VO-to-disk 
	 * API with automatic name-sorted indexing.  
	 */
	public class AbstractProxy extends Proxy
	{
		
		// Extension for files
		private const FILE_EXT:String = ".xml";
		
		// Index filename
		private const INDEX:String    = "index"+FILE_EXT;
		
		// The folder to access. Set by connectToFolder()
		private var folder:File;
		
		/**
		 * Constructor. 
		 * 
		 * Name provided by subclass, data property 
		 * (used for the index of stubbed VOs) is 
		 * initialized to ArrayCollection.
		 */
		public function AbstractProxy( name:String )
		{
			super( name, new ArrayCollection() );
		}
		
		/**
		 * The name-sorted index of stubbed VOs.
		 */
		public function get index():ArrayCollection
		{
			return data as ArrayCollection;
		}
					
		/**
		 * Abstract factory method to create stubbed VOs
		 * for the index. Must be overridden in subclass 
		 * and return a stubbed VO of the appropriate type
		 * for inclusion in the index.
		 */
		protected function getVoInstance( xml:XML ):ValueObject
		{
			return null;
		}
		
		/**
		 * Sets the protected folder property. 
		 * 
		 * Also, if the folder doesn't exist, it is 
		 * created and an empty index written. Otherwise, 
		 * the index is read from the existing folder.
		 */
		protected function connectToFolder( name:String ):void
		{
			folder = File.applicationStorageDirectory.resolvePath( name );
			if ( !folder.exists ) {
				folder.createDirectory();
				writeIndex();
			} else {
				readIndex();
			}
		}
		
		/**
		 * Write the value object's XML property to a file
		 * and update the index on disk and in memory.
		 */
		protected function writeVO( vo:ValueObject ):void
		{
			var file:File = getFile( vo );
			writeFile( file, vo.xml );
			updateIndex( vo );
		}

		/**
		 * From the stub passed in, read the corresponding VO 
		 * from the disk and return the XML.
		 */
		protected function readVO( voStub:ValueObject ):XML
		{
			var file:File = getFile( voStub );
			return XML ( readFile( file ) );
		}

		/**
		 * From the VO passed in, delete the corresponding VO
		 * from the disk and update the index.
		 */
		protected function deleteVO( vo:ValueObject ):void
		{
			var file:File = getFile( vo );
			if ( file.exists ) file.deleteFile(); 
			updateIndex( vo, true );
		}
		
		/**
		 * Read the index XML from disk and create the 
		 * in memory index of stubbed VOs.
		 */ 
		private function readIndex():void
		{
			// read in the index XML
			var file:File = folder.resolvePath( INDEX );
			var xml:XML = readFile( file );
			
			// build the index from the XML
			var ac:ArrayCollection =  new ArrayCollection();
			for each ( var ix:XML in xml.children() )
			{
				var vo:ValueObject = getVoInstance( ix );
				ac.addItem( vo );
			}
			
			// Sort the index by name 
			var sort:Sort = new Sort();
			var field:SortField = new SortField( 'name' );
			sort.fields = [ field ];
			ac.sort = sort;
			ac.refresh();
			
			// set the index as the proxy data
			setData( ac );
		}
		
		/**
		 * Read the in memory index of stubbed VOs to 
		 * the index XML file on disk. 
		 */ 
		private function writeIndex():void
		{
			var file:File = folder.resolvePath( INDEX );
			var xml:XML = <index></index>;
			var ivo:ValueObject;
			for ( var i:int=0; i < index.length; i++ ){
				ivo = ValueObject( index.getItemAt(i) );
				xml.appendChild( ivo.getStub() );
			}
			writeFile( file, xml );
		}
		
		/**
		 * Update the in memory index of stubbed VOs, replacing or 
		 * removing the stubbed VO passed in, sort the index, and  
		 * write the index XML file on disk. 
		 */ 
		private function updateIndex( vo:ValueObject, removing:Boolean=false ):void
		{
			// Find and remove existing item if present
			for ( var i:int=0; i < index.length; i++ ){
				var ivo:ValueObject = ValueObject( index.getItemAt(i) );
				if ( ivo.uid == vo.uid ) {
					index.removeItemAt( i );
					break;
				}
			}
			
			// add the new or updated item to index if not removing
			if ( ! removing ) index.addItem( getVoInstance( vo.getStub() ) );
			
			// re-sort the index
			index.refresh();
			
			// persist the index
			writeIndex();
		}	
		
		/**
		 * Write xml to a file.
		 */
		private function writeFile( file:File, xml:XML ):void
		{
			var stream:FileStream = new FileStream()
			stream.open( file, FileMode.WRITE );	
			stream.writeUTFBytes( xml );
			stream.close();
		}
		
		/**
		 * Read XML from a file and return it.
		 */
		private function readFile( file:File ):XML
		{
			var stream:FileStream = new FileStream()
			stream.open( file, FileMode.READ );	
			var xml:XML = XML( stream.readUTFBytes( stream.bytesAvailable ) );
			stream.close();
			return xml;
		}
		
		/**
		 * Get the corresponding File object for a given VO.
		 */
		private function getFile( vo:ValueObject ):File
		{
			var filename:String = vo.uid + FILE_EXT;
			return folder.resolvePath( filename );
		}			
	}
}