package com.futurescale.sa.model.proxy
{
	import com.futurescale.sa.model.enum.StoryTypeEnum;
	import com.futurescale.sa.model.vo.StoryVO;
	import com.futurescale.sa.model.vo.ValueObject;
	
	import org.puremvc.as3.interfaces.IProxy;
	import org.puremvc.as3.patterns.proxy.Proxy;
	
	/**
	 * The StoryProxy provides persistence,
	 * indexing, and caching of StoryVOs
	 */
	public class StoryProxy extends AbstractProxy
	{
		// Registered name
		public static  const NAME:String   		 = "StoryProxy";
		
		// Filesystem folder name for storage
		private static const FOLDER:String 		 = "story";

		// Notes sent from this Proxy
		public static const STORY_ADDED:String   = NAME+"/"+FOLDER+"/added";
		public static const STORY_SAVED:String   = NAME+"/"+FOLDER+"/saved";
		public static const STORY_DELETED:String = NAME+"/"+FOLDER+"/deleted";
		
		// Known Collaborators
		private var milieuProxy:MilieuProxy;
		private var castProxy:CastProxy;
		
		/**
		 * Constructor. 
		 * 
		 * The StoryProxy collaborates with the 
		 * MilieuProxy and CastProxy to ensure
		 * consistency of the domain data. Requiring 
		 * them on the constructor both ensures they 
		 * will be present and encapsulates their 
		 * relationship. 
		 */
		public function StoryProxy( milieuProxy:MilieuProxy, castProxy:CastProxy )
		{
			super( NAME );

			this.milieuProxy = milieuProxy;
			this.castProxy	 = castProxy; 
		}
		
		/**
		 * Called by the framework when the Proxy is registered.
		 */
		override public function onRegister():void
		{
			connectToFolder( FOLDER );
		}

		/**
		 * Load a Story.
		 * 
		 * If Story is already cached from having been 
		 * previously loaded or added, the cached VO will be 
		 * returned, otherwise it will be loaded and cached first.
		 * 
		 * Optionally allows forced re-caching from disc 
		 * (as when user changes are discarded). 
		 */
		public function loadStory( storyStub:StoryVO, recache:Boolean = false ):StoryVO
		{
			// Optionally force loading from disc and recaching 
			if ( recache ) facade.removeProxy( storyStub.uid );
			
			var cacheProxy:IProxy; 
			if ( facade.hasProxy( storyStub.uid ) ) {
				cacheProxy = facade.retrieveProxy( storyStub.uid );
			} else {
				var story:StoryVO = new StoryVO( readVO( storyStub ) );
				cacheProxy = new Proxy( story.uid, story );
				facade.registerProxy( cacheProxy );
			}
			return cacheProxy.getData() as StoryVO;
		}
		
		/**
		 * Add a Story.
		 * 
		 * Story is written to the filesystem and cached in memory
		 * with a transient Proxy instance, so that it doesn't have
		 * to be loaded again later to be used. Also this method
		 * populates the Story down to the first draft.
		 */
		public function addStory( story:StoryVO ):void
		{
			// Cache Story
			facade.registerProxy( new Proxy( story.uid, story ) );

			// If this story is part of a series, it will already
			// have a cast and milieu, otherwise create them.
			if ( story.cast == null ) story.cast = castProxy.createCast( story.name );
			if ( story.milieu == null ) story.milieu = milieuProxy.createMilieu( story.name );
			
			// Populate story 
			switch ( story.type ) 
			{
				case StoryTypeEnum.SIMPLE:
					story.getNewScene();
					break;
				
				case StoryTypeEnum.NORMAL:
					story.getNewChapter();
					break;
				
				case StoryTypeEnum.COMPLEX:
					 story.getNewPart();
					break;
			}
			
			// Save to the filesystem, updating the index
			writeVO( story ); 
			
			// Notify observers 
			sendNotification( STORY_ADDED, story );
		}
		
		/**
		 * Save a Story.
		 */
		public function saveStory( story:StoryVO ):void
		{
			// save to the filesystem, updating the index
			writeVO( story );
			
			// Notify observers 
			sendNotification( STORY_SAVED, story );
		}
		
		/**
		 * Delete a Story.
		 * 
		 * Removes the transient Proxy holding the cached
		 * Story if present, then removes the Story from 
		 * filesystem. Index will be automatically updated.  
		 */
		public function deleteStory( story:StoryVO ):void
		{
			// Remove the cached story
			if ( facade.hasProxy( story.uid ) ) {
				facade.removeProxy( story.uid );
			}
			
			// remove cast and milieu if not part of a series
			if ( ! story.isEpisode ) {
				castProxy.deleteCast( story.cast );
				milieuProxy.deleteMilieu( story.milieu );
			}
			
			// remove from filesystem and update index
			deleteVO( story );

			// Notify observers with stub so that story is GC'd
			sendNotification( STORY_DELETED, new StoryVO( story.getStub() ) );
		}		
		
		/**
		 * VO Factory Method. Required for indexing in superclass.
		 */
		override protected function getVoInstance( xml:XML ):ValueObject
		{
			return new StoryVO( xml );
		}
	}
}