package com.futurescale.sa.model.proxy
{
	import com.futurescale.sa.model.vo.SeriesVO;
	import com.futurescale.sa.model.vo.StoryVO;
	import com.futurescale.sa.model.vo.ValueObject;
	
	import org.puremvc.as3.interfaces.IProxy;
	import org.puremvc.as3.patterns.proxy.Proxy;
	
	/**
	 * The SeriesProxy provides persistence,
	 * indexing, and caching of SeriesVOs
	 */
	public class SeriesProxy extends AbstractProxy
	{
		// Registered name
		public  static const NAME:String   		  = "SeriesProxy";
		
		// Filesystem folder name for storage
		private static const FOLDER:String 		  = "series";

		// Notes sent from this Proxy
		public static const SERIES_ADDED:String   = NAME+"/"+FOLDER+"/added";
		public static const SERIES_SAVED:String   = NAME+"/"+FOLDER+"/saved";
		public static const SERIES_DELETED:String = NAME+"/"+FOLDER+"/deleted";
		
		// Known Collaborators
		private var storyProxy:StoryProxy;
		private var milieuProxy:MilieuProxy;
		private var castProxy:CastProxy;
		
		/**
		 * Constructor. 
		 * 
		 * The SeriesProxy collaborates with the StoryProxy, 
		 * MilieuProxy and CastProxy to ensure
		 * consistency of the domain data. Requiring 
		 * them on the constructor both ensures they 
		 * will be present and encapsulates their 
		 * relationship. 
		 */
		public function SeriesProxy( storyProxy:StoryProxy, milieuProxy:MilieuProxy, castProxy:CastProxy )
		{
			super( NAME );
			
			this.storyProxy	 = storyProxy;
			this.milieuProxy = milieuProxy;
			this.castProxy	 = castProxy; 
		}
		
		/**
		 * Called by the framework when the Proxy is registered.
		 */
		override public function onRegister():void
		{
			connectToFolder( FOLDER );
		}
		
		/**
		 * Load a Series.
		 * 
		 * If Series is already cached from having been 
		 * previously loaded or added, the cached VO will be 
		 * returned, otherwise it will be loaded and cached first.
		 * The index will be automatically updated. 
		 * 
		 * Optionally allows forced re-caching from disc 
		 * (as when user changes are discarded). 
		 */
		public function loadSeries( seriesStub:SeriesVO, recache:Boolean = false ):SeriesVO
		{
			if ( recache ) facade.removeProxy( seriesStub.uid );
			
			var cacheProxy:IProxy; 
			if ( facade.hasProxy( seriesStub.uid ) ) {
				cacheProxy = facade.retrieveProxy( seriesStub.uid );
			} else {
				var series:SeriesVO = new SeriesVO( readVO( seriesStub ) );
				cacheProxy = new Proxy( series.uid, series );
				facade.registerProxy( cacheProxy );
			}
			return cacheProxy.getData() as SeriesVO;
		}
		
		/**
		 * Add a Series.
		 * 
		 * Series is written to the filesystem and cached in memory
		 * with a transient Proxy instance, so that it doesn't have
		 * to be loaded again later to be used.
		 * 
		 * Cast and Milieu will be created, cached, saved and stubs
		 * added to the 
		 */
		public function addSeries( series:SeriesVO ):void
		{
			// Cache the Series
			facade.registerProxy( new Proxy( series.uid, series ) );
			
			// Create the Cast and Milieu
			series.cast = castProxy.createCast( series.name );
			series.milieu = milieuProxy.createMilieu( series.name );
			
			// Populate series
			if (series.useSeasons) series.getNewSeason();
			
			// Save the Series and update index
			writeVO( series );
			
			// Notify observers
			sendNotification( SERIES_ADDED, series );
		}
		
		/**
		 * Save a Series.
		 */
		public function saveSeries( series:SeriesVO ):void
		{
			// Save the Series and update index
			writeVO( series );
			
			// Notify observers
			sendNotification( SERIES_SAVED, series );
		}
		
		/**
		 * Delete a Series.
		 * 
		 * Removes the transient Proxy holding the cached
		 * Series if present, then removes the Series from 
		 * filesystem. Index will be automatically updated.  
		 * 
		 * Also removes Cast, Milieu, and all Episodes. 
		 */
		public function deleteSeries( series:SeriesVO ):void
		{
			// Remove the cached Series
			if ( facade.hasProxy( series.uid ) ) {
				facade.removeProxy( series.uid );
			}
			
			// Delete Cast and Milieu
			castProxy.deleteCast( series.cast );
			milieuProxy.deleteMilieu( series.milieu );
			
			// Delete Episodes
			var episode:StoryVO;
			for each ( episode in series.stories ) {
				storyProxy.deleteStory( episode );
			}
			
			// Delete Series from filesystem and update index
			deleteVO( series );
			
			// Notify Observers with stub so Series is GC'd
			sendNotification( SERIES_DELETED, new SeriesVO( series.getStub() ) );
		}
						
		/**
		 * VO Factory Method. Required for indexing in superclass.
		 */
		override protected function getVoInstance( xml:XML ):ValueObject
		{
			return new SeriesVO( xml );
		}

	}
}